<?php
/* 
	This is phpSitemapNG, a php script that creates your personal google sitemap
	It can be downloaded from http://enarion.net/google/
	License: GPL
	
	Tobias Kluge, enarion.net
*/

class FilesystemHandler {
	var $files = array();
	var $forbidden_dir = array();
	var $forbidden_files = array();
	
	var $filesystem_base = '';
	var $directory_offset = '';
	var $cur_item = 0;	
	 
	/**
	 * public constructor, set initial values
	 */
    function FilesystemHandler($filesystem_base, $directory_offset = '') {
		$this->filesystem_base = $filesystem_base;
		$this->directory_offset = $directory_offset; 
    }    
    
    /**
     * returns number of files
     */
    function size() {
    	return count($this->files);
    }
    
    /**
     * returns true when the current item is not the last item
     * behaves like in java
     */
    function hasNext() {
    	if ($this->size() > $this->cur_item) return true;
    	return false;
    }
    
    /**
     * returns the current item
     * behaves like in java
     */    
    function getNext() {
    	if ($this->hasNext()) {
    		$tmp = $this->files[$this->cur_item];
    		$this->cur_item++;
    		return $tmp;
    	}
    	return null;    		
    }
    
    /**
     * set list of forbidden directories
     */
    function setForbiddenDirectories($directories = array()) {
    	$this->forbidden_dir = $directories;	
    }
	
	/**
	 * set list of forbidden files
	 */
	function setForbiddenFiles($files = array()) {
		$this->forbidden_files = $files;	
	}    
	
	/**
	 * scans filesystem for files that match the setted conditions
	 * 
	 * return number of files that have been found
	 */
    function start() {
		$this->files = $this->_getFiles($this->filesystem_base, "", $this->directory_offset);
		return count($this->files);
    }
    
    /**
     * return last modification time
     */
    function getLastModificationTime($filename) {
    	$lastmod = filemtime($filename);
    	// if filemtime failed (for any reason), set it to current time
		if (!((!is_null($lastmod)) && is_integer($lastmod) && $lastmod > 0)) $lastmod = time(); 
    	
		return $lastmod;
    	
    }
  
  
/* some private functions */
	/* simple compare function: equals */
  	function _arr_contains($key, $array) {
	  if (is_array($array) && count($array) > 0) {
	    foreach ($array as $val) {
		  	if ($key == $val) {
				return true;
			}
	    }
	  }
	  return false;
	}

	/* better compare function: contains */
	function _fl_contains($key, $array) {
	  if (is_array($array) && count($array) > 0) {
	    foreach ($array as $val) {
		  $pos = @strpos($key, $val);
		  if ($pos === FALSE) continue;
		  return true;
	    }
	  }
	
	  return false;
	}

	/* this function changes a substring($old_offset) of each array element to $offset */
	function _changeOffset($array, $old_offset, $offset) {
	  $res = array();
	  if (is_array($array) && count($array) > 0) {
	    foreach ($array as $val) {
	      $res[] = str_replace($old_offset, $offset, $val);
	    }
	  }
	  return $res;
	}

	/* this walks recursivly through all directories starting at page_root and
	   adds all files that fits the filter criterias */
	// taken from Lasse Dalegaard, http://php.net/opendir
	function _getFiles($directory, $directory_orig = "", $directory_offset="") {
	   if ($directory_orig == "") $directory_orig = $directory;
	
	   if($dir = @opendir($directory)) {
	       // Create an array for all files found
	       $tmp = Array();
	
	       // Add the files
	       while($file = readdir($dir)) {
				// Make sure the file exists
				if($file != "." && $file != ".." && $file[0] != '.' ) { 
				   // If it's a directiry, list all files within it
					 if(is_dir($directory . "/" . $file)) {
						$disallowed_abs = $this->_fl_contains($directory."/".$file.'/', $this->forbidden_dir); // handle directories with pathes
						$disallowed = $this->_arr_contains($file.'/', $this->forbidden_dir); // handle directories only without pathes
						if ($disallowed || $disallowed_abs) continue;
					    $tmp2 = $this->_changeOffset($this->_getFiles($directory . "/" . $file, $directory_orig, $directory_offset), $directory_orig, $directory_offset);
					    if(is_array($tmp2)) {
					        $tmp = array_merge($tmp, $tmp2);
					    }
					} else {  // files
						if ($this->_fl_contains($file, $this->forbidden_files)) continue;
				    	array_push($tmp, str_replace($directory_orig, $directory_offset, $directory."/".$file));
					}
				}
	       }
	
	       // Finish off the function
	       closedir($dir);
	       return $tmp;
	   }
	}
    
}
?>