<?php
/* 
	This is phpSitemapNG, a php script that creates your personal google sitemap
	It can be downloaded from http://enarion.net/google/
	License: GPL
	
	Tobias Kluge, enarion.net
*/


/**
 * misc functions
 */

function debug($param, $msg = '') {
	global $SETTINGS, $LAYOUT;

	if ($SETTINGS[debug] === true || $_SESSION[debug]=== true) {
		if ($param == "" && $msg = "")
			return;
		if (is_array($param)) {
			$LAYOUT->addDebug(arrToStringReadable($param, "<br>\n"),$msg);
		} else {
			$LAYOUT->addDebug($param, $msg);
		}
	}
}

/**
 * returns a filehandle if file is accessable
 */
function openFile($filename, $writable = false) {
	global $openFile_error;
	$openFile_error = "";
	// check if file exists
	if (!file_exists($filename)) {
		$openFile_error = "File $filename does not exist";
		return false;
	}
	// check if file is accessable
	if (!is_readable($filename)) {
		$openFile_error = "File $filename is not readable";
		return false;
	}
	if ($writable && !is_writable($filename)) {
		$openFile_error = "File $filename is not writable";
		return false;
	}

	$accessLevel = 'r';
	if ($writable === true) {
		$accessLevel = 'w';
	}

	$filehandle = fopen($filename, $accessLevel);
	if ($filehandle === FALSE) {
		$openFile_error = "File $filename could not be opened, don't know why";
		@ fclose($filehandle);
		return false;
	}
	return $filehandle;
}

/**
 * start session
 * read settings from session
 * set and return action
 */
function init() {
	global $SETTINGS, $_REQUEST, $LAYOUT;

	session_start();

	// set layout engine
	$LAYOUT = new LayoutEngine("phpSitemapNG");
	$LAYOUT->addContentFooter('<div align="center"><p>Copyright by enarion.net. This script is licensed under GPL and can be downloaded from
					<a target="_blank" href="http://enarion.net/google/">enarion.net/google/</a></p></div>');
	$LAYOUT->setTitle("create your personal google sitemap file");
	$LAYOUT->setCharSet("iso-8859-1");
	$LAYOUT->addCss('.history, .required { background-color:#E0E0E0; }');
	$LAYOUT->addCss('Label {color:#000099; font-weight: bold; }');
	$LAYOUT->addCss('h1,h2,h3 {color:#000099; }');
	$LAYOUT->addCss('.error {color:#cc0000; font-weight: bold; }');
	$LAYOUT->addCss('.warning {color:#000000; font-weight: italic; }');
	$LAYOUT->addCss('.info {color:#000000; font-weight: normal; }');
	$LAYOUT->addCss('.success {color:#009900; font-weight: bold; }');
	$LAYOUT->addCss('body {color:#000000; font-family:helvetica; background-color:#ebb150; }');
	
	if ($_REQUEST[debug] === "on") {
		$SETTINGS[debug] = true;
		debug($SETTINGS[debug], "Debug on: ");
		$_SESSION[debug] = true;
	}
	elseif ($_REQUEST[debug] === "off") {
		debug($SETTINGS[debug], "Debug off: ");
		$SETTINGS[debug] = false;
		$_SESSION[debug] = false;
	}

	$SETTINGS = array_merge($SETTINGS, $_SESSION[settings]);
	debug($SETTINGS, "Merged settings");

	debug($SETTINGS[state], "last state");
	$action = $_REQUEST[action];
	$SETTINGS[state] = $action;
	debug($SETTINGS[state], "current state");

	$SETTINGS[executed][$action] = true;

	$LAYOUT->addContentHeader('<a href="'.$SETTINGS[script].'?action=setup">Setup</a>');
	if ($SETTINGS[executed][getSettings] == true)
		$LAYOUT->addContentHeader('<a href="'.$SETTINGS[script].'?action=parse">Start scan</a>');
	if (file_exists($SETTINGS[sitemap_file]) && ( filesize($SETTINGS[sitemap_file]) > 0))
		$LAYOUT->addContentHeader('<a href="'.$SETTINGS[sitemap_url].'" target="_blank">View sitemap </a>');


	return $action;
}

/**
 * overwrite existing filelist-file
 */
function resetFiles() {
	global $SETTINGS;
	// write empty settings to file	
	storeSettings(array (), $SETTINGS[files_file], "FILES");
	// all settings are okay for now, so go on
	return '<p>Your filelist '.$SETTINGS[files_file].' is reseted now!<br/>'.'<a href="'.$script.'">You can now create your personal google sitemap</a></p>';
}

/**
 * checks a given filename if it exists and is writable
 * returns null, if okay; otherwise the error message
 */
function checkFile($filename) {
	$file = @ fopen($filename, "r");
	// check if writable
	$msg = "";
	if (!file_exists($filename)) {
		$msg = "File ".$filename." does not exist; create file and set permission with chmod to 0666";
	}
	elseif (!is_writable($filename)) {
		$msg = "File ".$filename." does exist but cannot be written; change permission with chmod to 0666";
	}
	elseif ($file === FALSE) {
		$msg = "Error while opening ".$filename." for write access. Check existence and permission of file!";
	}
	@ fclose($file);
	return $msg;
}

function getDateTimeISO($timestamp) {
	return date("Y-m-d\TH:i:s", $timestamp) . substr(date("O"),0,3) . ":" . substr(date("O"),3);
}

function getFrequency($lastmod) {
	// set changefreq
	$age = time() - $lastmod;
	$change_freq = "monthly"; // default value
	if ($age < 10) {
		$change_freq = "always";
	} elseif ($age < 60*60) {
		$change_freq = "hourly";
	} elseif ($age < 60*60*24) {
		$change_freq = "daily";
	} elseif ($age < 60*60*24*7) {
		$change_freq = "weekly";
	} elseif ($age < 60*60*24*31) { // longest month has 31 days
		$change_freq = "monthly";
	} elseif ($age < 60*60*24*365) {
		$change_freq = "yearly";
	} else {
		$change_freq = "never";
	}
	return $change_freq;	
}

/* write settings to file */
function storeSettings($SETTINGS, $filename, $keyname) {
	global $openFile_error;

	$file = openFile($filename, true);

	if ($file === false)
		return $openFile_error;

	fputs($file, "<?\n");
	foreach ($SETTINGS as $key => $val) {
		if (is_array($val)) {
			foreach ($val as $key2 => $val2) {
				fputs($file, '$'.$keyname."['".stringToVariableName($key)."']['".stringToVariableName($key2)."'] = '".stringToVariableName($val2)."';\n");
			}
			continue;
		}
		fputs($file, '$'.$keyname."['".stringToVariableName($key)."'] = '".stringToVariableName($val)."';\n");
	}
	fputs($file, "?>\n");
	fclose($file);

	return null;
}

/* remove maybe existing ' in string */
function stringToVariableName($str) {
	return ((!get_magic_quotes_gpc()) ? addslashes($str) : $str);
}
function variableNameToString($var_name) {
	if(get_magic_quotes_gpc()) {
		return stripslashes($var_name);
	} else {
		$res = $var_name;
		if (substr($res, 0, 1) == "'")
			$res = substr($res, 1);
		if (substr($res, strlen($res) - 1) == "'")
			$res = substr($res, 0, strlen($res) - 1);
		return $res;
	}
}

/* explodes the array for given deliminator and returns a correct array */
function toArray($str, $delim = "\n") {
	$res = array ();
	$res = explode($delim, $str);

	for ($i = 0; $i < count($res); $i ++) {
		$res[$i] = trim($res[$i]);
	}

	return $res;
}

/* returns a string of all entries of array with delim */
function arrToStringReadable($array, $delim) {
	$res = "";
	if (is_array($array)) {
		$i = 0;
		foreach ($array as $key => $val) {
			if (is_array($val)) {
				$res .= "$key: ".arrToString($val, $delim);
			} else {
				$res .= "$key: $val";
			}
			if ($i < (count($array) - 1))
				$res .= $delim;
			$i ++;
		}
	}
	return $res;
}

/* returns a string of all entries of array with delim */
function arrToString($array, $delim = "\n") {
	$res = "";
	if (is_array($array)) {
		for ($i = 0; $i < count($array); $i ++) {
			$res .= $array[$i];
			if ($i < (count($array) - 1))
				$res .= $delim;
		}
	}
	return $res;
}

/**
 * display page 
 */
function msg($msg, $title = "create your personal google sitemap file") {
	global $SETTINGS;
	echo '
	<html>
	<head>
		<title>phpSitemap: '.$title.' -- powered by enarion.it</title>
		<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
		<style type="text/css">
		<!--
			.history, .required { background-color:#E0E0E0; }
			Label {color:#000099; font-weight: bold; }
			h1,h2,h3 {color:#000099; }
			.failure {color:#cc0000; font-weight: bold; }
			.warning {color:#0099; font-weight: italic; }		
			.okay {color:#009900; font-weight: normal; }		
			body {color:#000000; font-family:helvetica; background-color:#ebb150; }
		-->
	</style>
	</head>
	<body>
	<h1>'.$title.'</h1>
	<p><a href="'.$SETTINGS[script].'?action=setup">Setup</a></p>
	';
	if ($SETTINGS[executed][getSettings] == true)
		echo '<p><a href="'.$SETTINGS[script].'?action=parse">Start scan</a></p>'."\n";
	if (file_exists($SETTINGS[sitemap_file]) && (@ filesize($SETTINGS[sitemap_file]) > 0))
		echo '<p><a href="'.$SETTINGS[sitemap_url].'" target="_blank">View sitemap </a></p>'."\n";

	echo '
	<div align="center">
	'.$msg.'
	</div>
	<p>'.$SETTINGS[debug_output].'</p>
	<div align="center"><p>Copyright by enarion.net. This script is licensed under GPL and can be downloaded under
			<a target="_blank" href="http://enarion.net/google/">enarion.net/google/</a></p></div>
	</body></html>
	';
}
?>