<?php
/* 
	This is phpSitemapNG, a php script that creates your personal google sitemap
	It can be downloaded from http://enarion.net/google/
	License: GPL
	
	Tobias Kluge, enarion.net
*/

/**
 * parse filesystem for files
 */
function parseFilesystem($displayResultForEditing = true) {
	global $SETTINGS, $LAYOUT;
	// search for files on local file system
	if(is_null($SETTINGS[page_root]) || $SETTINGS[page_root] == "") {
		$LAYOUT->addError("","Page_root not set");
		debug("", "parseFilesystem() - page_root is not set!");
		return;
	}
	$fsh = new FilesystemHandler($SETTINGS[page_root]);
	$fsh->setForbiddenDirectories($SETTINGS[disallow_dir]);
	$fsh->setForbiddenFiles($SETTINGS[disallow_file]);

	if ($fsh->start() > FILE_LIMIT) {
		$LAYOUT->addWarning('Will not display the result for editing.<br/>' .
					'If you want to edit the sitemap, please edit the generated file directly.',
					'Too much files('.$fsh->size().')');
		$msg .= writeSitemap($fsh); 
	} else {
		$msg .= displaySitemapEdit($fsh);
	}
	return $msg;
}

function displaySitemapEdit($fsh) {
	global $SETTINGS, $LAYOUT;
	$LAYOUT->setTitle("Result of scan");
	$title = 'Found '. $fsh->size() .' files';

	$msg .= '<form action="'.$SETTINGS[script].'" method="post"><input type="submit" name="submit" value="Create file">';
	$msg .=	'<input type="hidden" name="action" value="writeSitemapUserinput">' . "\n";		
	$msg .= "<table><th>Number</th><th>Filename</th><th>Lastmodification</th><th>Change frequency</th><th>Relative Priority</th>\n";

	// import file list
	@include_once($SETTINGS[files_file]);
	
	if (!is_null($FILES)) {
		$FILES_CACHE = array();
		foreach ($FILES as $numb => $arr) {
			$file = $arr[file];
			$FILES_CACHE[$file]['lastmod'] = variableNameToString($arr['lastmod']);
			$FILES_CACHE[$file]['changefreq'] = variableNameToString($arr['changefreq']);
			$FILES_CACHE[$file]['priority'] = variableNameToString($arr['priority']);
			$FILES_CACHE[$file]['enabled'] = variableNameToString($arr['enabled']);
		}				
	}
	debug($FILES_CACHE, "FILES_CACHE");
	
	$numb = 0;
	while($fsh->hasNext()) {
		$file = stringToVariableName($fsh->getNext());
		$file_url = urlencode($file);

		// default: file is enabled and will be handled
		$enabled = true;
		
		$lastmod = $fsh->getLastModificationTime($SETTINGS[page_root].$file);
		$last_modification =  getDateTimeISO($lastmod);

		if($SETTINGS[changefreq_fixed] != "") {
			$change_freq = $SETTINGS[changefreq_fixed];
		} else {
			$change_freq = getFrequency($lastmod);
		}

		// TODO maybe calculate priority depending on #of links, hierarchie level, ...
		$priority = $SETTINGS[priority];	
		
		// get maybe already existing values from filelist file
		if (!is_null($FILES_CACHE[$file])) {
			$enabled = $FILES_CACHE[$file]['enabled'];
			$change_freq = $FILES_CACHE[$file]['changefreq'];
			$priority = $FILES_CACHE[$file]['priority'];
			$history = 'class="history"';
		}

		if (!is_null($enabled)&& strlen($enabled) > 0) {
			 $enabled = "checked";
		}
		$msg .= "<tr $history>\n" .
			'<td class="number"><input type="checkbox" '.$enabled.' name="FILE['.$numb.'][enabled]" value="true">'.$numb.'</td>'."\n" .
			'<td class="filename"><input type="hidden" name="FILE['.$numb.'][file]" value="'.$file_url.'">'.$file."</td>\n" .
			'<td class="lastmod"><input class="lastmod" type="text" name="FILE['.$numb.'][lastmod]" value="'.$last_modification.'" size="24"></td>'."\n" .
			'<td class="changefreq"><select name="FILE['.$numb.'][changefreq]" size="1"><option>'.$change_freq.'</option>' .
				"<option>always</option>" .
				"<option>hourly</option>" .
				"<option>daily</option>" .
				"<option>weekly</option>" .
				"<option>monthly</option>" .
				"<option>yearly</option>" .
				"<option>never</option></select></td>\n" .
			'<td class="priority"><select name="FILE['.$numb.'][priority]" size="1"><option>'.$priority."</option>" .
				"<option>1.0</option>" .	
				"<option>0.9</option>" .
				"<option>0.8</option>" .
				"<option>0.7</option>" .
				"<option>0.6</option>" .
				"<option>0.5</option>" .
				"<option>0.4</option>" .
				"<option>0.3</option>" .
				"<option>0.2</option>" .
				"<option>0.1</option>" .
				"<option>0.0</option>" .
				"</select></td>\n" .
			"</tr>\n";
		$numb++;
	}
	$msg .= '<tr class="history"><td colspan="5">A row with this background means that this file is already stored in your local cached filelist</td></tr>'."\n";	
	$msg .= "</table></form>\n";
	$LAYOUT->addText($msg, $title);
}


function writeSitemap($fsh) {
	global $SETTINGS, $openFile_error, $LAYOUT;	
	$LAYOUT->setTitle("Writing sitemap");

	// import file list
	@include_once($SETTINGS[files_file]);
	if (!is_null($FILES)) {
		$FILES_CACHE = array();
		foreach ($FILES as $numb => $arr) {
			$file = $arr[file];
			$FILES_CACHE[$file]['lastmod'] = variableNameToString($arr['lastmod']);
			$FILES_CACHE[$file]['changefreq'] = variableNameToString($arr['changefreq']);
			$FILES_CACHE[$file]['priority'] = variableNameToString($arr['priority']);
			$FILES_CACHE[$file]['enabled'] = variableNameToString($arr['enabled']);
		}				
	}
	debug($FILES_CACHE, "FILES_CACHE");
	

	$gsg = new GsgXml($SETTINGS[website]);
	
	while($fsh->hasNext()) {
		$file = $fsh->getNext();
		debug($file, "Adding file to sitemap");
//		$file = stringToVariableName($file);
		$enabled = true;

		$lastmod = $fsh->getLastModificationTime($SETTINGS[page_root].$file);
		$last_modification =  getDateTimeISO($lastmod);
	
		if($SETTINGS[changefreq_fixed] != "") {
			$change_freq = $SETTINGS[changefreq_fixed];
		} else {
			$change_freq = getFrequency($lastmod);
		}
		// TODO maybe calculate priority depending on #of links, hierarchie level, ...
		$priority = $SETTINGS[priority];

		// get maybe already existing values from filelist file
		if (!is_null($FILES_CACHE[$file])) {
			$enabled = $FILES_CACHE[$file]['enabled'];
			$change_freq = $FILES_CACHE[$file]['changefreq'];
			$priority = $FILES_CACHE[$file]['priority'];
			$history = 'class="history"';
		}

		if (!is_null($enabled)&& strlen($enabled) > 0) {		
			if ($gsg->addUrl($file, true, $lastmod, false, $change_freq, $priority) === false) {
				debug("", "result of writeSitemap::addUrl (file $file) was false");	
			}
		} else {
			debug($file, 'File not enabled, so not written to sitemap file');
		}
	}
	
	$filehandle = openFile($SETTINGS[sitemap_file], true);
	if ($filehandle === false) {
		$LAYOUT->addError($openFile_error, 'Could not write sitemap');
		return false;
	}
	$xml = $gsg->output(true, false, false);

	fputs ($filehandle, $xml);
	fclose ($filehandle);
		
	$LAYOUT->addText('<form action="' . $SETTINGS[script] . '" method="post">' ."\n".
			'<input type="hidden" name="action" value="submitPageToGoogle">' . "\n".		
			'<input type="Submit" value="Submit to google" name="submit">' . "\n".
			'</form>' . "\n", 'Sitemap successfuly created and saved to <a href="'.$SETTINGS[sitemap_url].'" target="_blank">file</a>!');
	
	return true;	
}


/**
 * deprecated: handle create sitemap for given list of files
 */ 
function writeSitemapUserinput() {
	global $SETTINGS, $openFile_error, $_REQUEST, $LAYOUT;
	$LAYOUT->setTitle('Writing sitemap');
	
	$gsg = new GsgXml($SETTINGS[website]);

 // create the sitemap file
	$filesGot = $_REQUEST[FILE];
	$files = array();
	foreach ($filesGot as $key => $value) {
		$files[$key] = array();
		$files[$key][enabled] = ($value[enabled] == "true");
		$files[$key][file] = urldecode($value[file]);
		$files[$key][lastmod] = $value[lastmod];
		$files[$key][changefreq] = $value[changefreq];
		$files[$key][priority] = $value[priority];
	}
	
	if(count($files)>FILE_LIMIT) {
		$LAYOUT->addInfo('Filelist (for cached settings of files) cannot be stored. This means that your sitemap file has been successfuly created.
			But your settings for the file attributes(made a step ahead) have not been stored - so you have to set it up when you run this script the next time.',
			'Warning: Too much files');
	} else {		
		 $res = storeSettings($files, $SETTINGS[files_file], "FILES");
		 if (!is_null($res)) {
			$LAYOUT->addWarning($res, 'Filelist-Cache could not be written to file ' . $SETTINGS[files_file] . '!');
		} else {
			$LAYOUT->addSuccess('', 'Filelist-Cache written to file ' . $SETTINGS[files_file] . '!');
		}		 
	}
			
	// try to open xml file
	$filehandle = openFile($SETTINGS[sitemap_file], true);
	if ($filehandle === false) {
		$LAYOUT->addError($openFile_error, 'Error while writing sitemap');
		return false;
	}	
	// file is open, write xml to file
	$i = 0;
	foreach ($files as $numb => $file) {
		if ($file[enabled]== "") {
			debug($file[file], "not writing file");
			continue;
		} else {
			if ($gsg->addUrl($file[file], true, $file[lastmod], false, $file[changefreq], $file[priority]) === false) {
				$debug("", "result of writeSitemapUserinput::addUrl (file ".$file[file].") was false");
				continue;	
			}
			$i++;
		}
	}
	$xml = $gsg->output(true, false, false);
	
	fputs ($filehandle, $xml);
	fclose ($filehandle);	
		
	$LAYOUT->addSuccess('Exported ' . $i . ' entries to '.$SETTINGS[sitemap_url]. '<br />' .
				'View <a href="'.$SETTINGS[sitemap_url].'" target="_blank">sitemap file</a> in browser','Export successful');
	
	if ($numb > 50000) {
		$LAYOUT->addWarning('Not implemented: split result into files with only 50000 entries','Only 50000 entries are allowed in one sitemap file at the moment!');
	}
	
	$LAYOUT->addText('<form action="' . $SETTINGS[script] . '" method="post">' ."\n".
			'<input type="hidden" name="action" value="submitPageToGoogle">' . "\n".		
			'<input type="Submit" value="Submit to google" name="submit">' . "\n".
			'</form>', 'Submit sitemap to google');
	return true;	
	//header('Content-type: application/xml; charset="utf-8"',true);	
}


/**
 * submit page to google
 */
function submitPageToGoogle() { 
	global $SETTINGS, $LAYOUT;
	$LAYOUT->setTitle('Submit sitemap to google');

	$res = fopen("http://www.google.com/webmasters/sitemaps/ping?sitemap=".urlencode($SETTINGS[sitemap_url]),"r");
	if ($res === FALSE) {
		$LAYOUT->addError('', 'Error while submitting '.$SETTINGS[sitemap_url].'to google!');
	}
	
	$str = fread($res, 10000);
	fclose($res);
	$LAYOUT->addSuccess('Result was: <i>'.	strip_tags($str, '<br> <h2> <h1>')	. '</i>',
							 'Your sitemap file has been successfuly sent to google!');
	return true;
}
?>
