<?php
/* 
	This is phpSitemapNG, a php script that creates your personal google sitemap
	It can be downloaded from http://enarion.net/google/
	License: GPL
	
	Tobias Kluge, enarion.net
*/
 
/**
 * print settings
 */ 
function debugOutput() {
	global $SETTINGS, $LAYOUT;
	$LAYOUT->setTitle("Debug output");
 	// check existance and permissions of files:
	$files = $SETTINGS[config];
	$files[] = $SETTINGS[sitemap_file];
	$files[] = $SETTINGS[settings_file];
	$files[] = $SETTINGS[files_file];
	
	$LAYOUT->addDebug('', 'Version: '.VERSION);
	$msg = '';	
	foreach ($files as $file) {
		if ($file == "") continue;
		$msg .= "<p>File $file:<ul>";
		if (@file_exists($file) ) {
			$msg .= '<li class="okay">does exists</li>';
		} else {
			$msg .= '<li class="failure"">does not exists</li>';
		}
		if (is_readable($file)) {
			$msg .= '<li class="okay">is readable</li>';
		} else {
			$msg .= '<li class="failure">is not readable</li>';
		}			
		if (is_writable($file)) {
			$msg .= '<li class="okay">is writable</li>';
		} else {
			$msg .= '<li class="failure">is not writable</li>';
		}			
		if (is_executable($file)) {
			$msg .= '<li class="warning">is executable - not necessary</li>';
		} else {
			$msg .= '<li class="okay">is not executable</li>';
		}
		$filehandle = @fopen($file, 'r');
		if ($filehandle === FALSE) {
			$msg .= '<li class="failure">could not be opened for read access</li>';			
		} else {
			$msg .= '<li class="okay">could be opened for read access, looks good</li>';
		}
		@fclose($filehandle);
		$msg .="</ul></p>\n";
	}
	$LAYOUT->addDebug($msg, 'Files');
	
	// debug settings
	$msg = '';
	if (is_null($SETTINGS) && count($SETTINGS) == 0) {
		$msg .= '<div class="failure">No settings found!</div> Please configure phpSitemap and make shure, the result is stored!<br/>'."\n";
	} else {
		$msg .= "<ul>";
		foreach($SETTINGS as $key => $val) {
			$msg .= "<li>";
			if (is_array($val)) {
				$msg .= "$key:<br/><ul>";
				foreach($val as $key2 => $val2) {
					$msg .= "<li>".$key.'['.$key2.'] = '.$val2."</li>\n";
				}
				$msg .= "</ul>\n";
			} else {
				$msg .= "$key = $val";
			}
			$msg .= "</li>\n";
		}
		$msg .= "</ul>";
	} 
	$msg .= "</p>";
	$LAYOUT->addDebug($msg, 'Settings');
	

	@include_once($SETTINGS[files_file]);
	
	// debug files: number, first, last, random (?)
	$msg = "<p>Number of files: " . count($FILES) . "<br/>\n";
	if (is_null($FILES)) {
		$msg .= '<div class="failure">No files found!</div> There will be files when phpSitemap is configured and successfuly executed.<br/>'."\n";	
	} else {
		$it = array(0, round(count($FILES)/2), count($FILES)-1); 
		$keys = array_keys($FILES); 
		foreach ($it as $i) {
			$msg .= "<li>File $i - " . $keys[$i].":<ul>\n";
			if (is_array($FILES[$keys[$i]])) {
				foreach($FILES[$keys[$i]] as $key => $val) {
					$msg .= "<li>$key = $val</li>\n";
				}
			} else {
				$msg .= "<li>".$FILES[$keys[$i]]."</li>\n";
			}
			$msg .= "</ul></li>\n";
		}
	}
	$msg .= "</p></div>\n";
	$LAYOUT->addDebug($msg, 'Files in filecache file');	
	return true;
}

/**
 * show settings
 * if settings are empty, set them to usabe values 
 */
function viewSetup($resetSettings = false) {
	global $SETTINGS, $LAYOUT;
	$LAYOUT->setTitle('Edit settings');

	if($resetSettings === true) {
		$SETTINGS = array();
	}
		
	if (is_null($SETTINGS[page_root])) 		$SETTINGS[page_root] 	= $_SERVER[DOCUMENT_ROOT];
	if (is_null($SETTINGS[website])) 		$SETTINGS[website] 		= "http://".$_SERVER[HTTP_HOST];
	if (is_null($SETTINGS[sitemap_file])) 	$SETTINGS[sitemap_file] 	= $SETTINGS[page_root] . "/sitemap.xml";
	if (is_null($SETTINGS[sitemap_url])) 	$SETTINGS[sitemap_url] 		= $SETTINGS[website] . "/sitemap.xml";
	if (is_null($SETTINGS[priority])) 		$SETTINGS[priority] 		= 0.5;
	/* list of disallowed directory names */
	if (is_null($SETTINGS[disallow_dir])) {
		$SETTINGS[disallow_dir] = array();
		$SETTINGS[disallow_dir][] = "admin";
		$SETTINGS[disallow_dir][] = "include";
		$SETTINGS[disallow_dir][] = "inc";
		$SETTINGS[disallow_dir][] = "logs";
		$SETTINGS[disallow_dir][] = "cgi-bin";
	}

	/* list of disallowed file names */
	if (is_null($SETTINGS[disallow_file])) {		
		$SETTINGS[disallow_file] = array();
		$SETTINGS[disallow_file][] = ".xml";
		$SETTINGS[disallow_file][] = ".inc";
		$SETTINGS[disallow_file][] = ".old";
		$SETTINGS[disallow_file][] = ".save";
		$SETTINGS[disallow_file][] = ".txt";
		$SETTINGS[disallow_file][] = ".js";
		$SETTINGS[disallow_file][] = "~";
		$SETTINGS[disallow_file][] = ".LCK";
		$SETTINGS[disallow_file][] = ".zip";
		$SETTINGS[disallow_file][] = ".ZIP";
		$SETTINGS[disallow_file][] = ".CSV";
		$SETTINGS[disallow_file][] = ".csv";
		$SETTINGS[disallow_file][] = ".css";
		$SETTINGS[disallow_file][] = ".class";
		$SETTINGS[disallow_file][] = ".jar";
	}
	if ($SETTINGS[changefreq_fixed]== false && $SETTINGS[changefreq_dynamic]== false) {
		$SETTINGS[changefreq_dynamic] = true;	
	}
	$str_disallow_dir = arrToString($SETTINGS[disallow_dir]);
	$str_disallow_file = arrToString($SETTINGS[disallow_file]);

	$msg = '
	<form action="'.$SETTINGS[script].'" method="post">
	<input type="hidden" name="action" value="getSettings">		
	<fieldset style="padding: 10; width:500; border-color:#000099; border-width:2px; border-style:solid; ">
	<legend style="color:#000099;"><b>Current settings</b></legend>
	<table border="0" cellpadding="5" cellspacing="0" width="495">
	  <tr>
	  	<td width="250" valign="top"><label for="ipage_root" accesskey="R">Page root</label><br />
			<font size="-1">path on local file system of your server</font></td>
		<td width="240">
			<input class="required" type="Text" name="page_root" id="ipage_root" align="LEFT" size="50" value="'.
			 $SETTINGS[page_root] .'"/>
		</td>
	  </tr>	
	  <tr>
	  	<td width="250" valign="top"><label for="iwebsite" accesskey="W">Website</label><br />
			<font size="-1">the url of your website</font></td>
		<td width="240">
			<input class="required" type="Text" name="website" id="iwebsite" align="LEFT" size="50" value="'.$SETTINGS[website].'"/>
		</td>
	  </tr>
	  <tr>
	  	<td width="250" valign="top"><label for="idisallow_dir" accesskey="D">Disallowed directories</label><br />
			<font size="-1">this directories will NOT be searched for files and will not be added to site index; use line break to separate entries</font></td>
		<td width="240">
			<textarea name="disallow_dir" cols="40" rows="10" id="idisallow_dir">'.$str_disallow_dir.'</textarea>
		</td>
	  </tr>
	  <tr>
	  	<td width="250" valign="top"><label for="idisallow_file" accesskey="F">Disallowed file names</label><br />
			<font size="-1">files containing this will not be added to site index; use line break to separate entries</font></td>
		<td width="240">
			<textarea name="disallow_file" cols="40" rows="10" id="idisallow_file">'.$str_disallow_file.'</textarea>
		</td>
	  </tr>
	  <tr>
	  	<td width="250" valign="top"><label for="isitemap_file" accesskey="S">Sitemap url</label><br />
			<font size="-1">where to store sitemap file - relative to your document root; this must exist, be writetable and accessible for the google bot!</font></td>
		<td width="240">
			<input type="Text" name="sitemap_url" id="isitemap_url" align="LEFT" size="50" value="'.$SETTINGS[sitemap_url].'"/>
		</td>
	  </tr>	
	  <tr>
	  	<td width="250" valign="top"><label for="ipriority" accesskey="P">Priority</label><br />
			<font size="-1">from 0.0 to 1.0, default 0.5</font></td>
		<td width="240">
			<input type="Text" name="priority" id="ipriority" align="LEFT" size="50" value="'.$SETTINGS[priority].'"/>
		</td>
	  </tr>	
	  <tr>
	  	<td width="250" valign="top"><label for="ichangefreq" accesskey="C">Change frequency</label><br />
			<font size="-1">from 0.0 to 1.0, default 0.5</font></td>
		<td width="240">
			<input type="radio" '.(($SETTINGS[changefreq_dynamic] != "") ? 'checked' : '' ) . ' name="changefreq" value="changefreq_dynamic">Calculate change frequency from last modification date<br/>
			<input type="radio" '.(($SETTINGS[changefreq_fixed] != "") ? 'checked' : '' ) . ' name="changefreq" value="changefreq_fixed">Use this change frequency for every file(will not overwrite stored file information):
					<select name="changefreq_fixed" id="ichangefreq_fixed" size="1">
						<option>'.$SETTINGS[changefreq_fixed].'</option>
						<option>always</option>
						<option>hourly</option>
						<option>daily</option>
						<option>weekly</option>
						<option>monthly</option>
						<option>yearly</option>
						<option>never</option>
					</select><br/>
		</td>
	  </tr>	
	  <tr>
	    <td>&nbsp;</td>
		<td><input type="Submit" value="Submit Settings" name="getSettings">&nbsp;
<!--			<input type="Submit" value="Reset to initial values" name="submit">&nbsp;
			<input type="Submit" value="Reset filelist" name="submit"> -->
		</td>
	  </tr>		
	</table>
	</fieldset>
	</form>';
	
	$LAYOUT->addText($msg, 'Current settings');
	return true;
} 

/**
 * extract settings from passed user values
 */
function getSettings() {	
	global $_REQUEST, $SETTINGS, $LAYOUT;
	$LAYOUT->setTitle("Storing settings");
	
	// TODO check values we got from user
	$SETTINGS[website] = $_REQUEST[website];
	if (substr($SETTINGS[website],0,4) != "http") {
		// user hasn't put a http in front of the url of the website
		$SETTINGS[website] = "http://".$SETTINGS[website]; 
	} 
	$SETTINGS[page_root] = $_REQUEST[page_root];
	
	// sitemap is the file after the website-part of the sitemap_url
	$sitemap = substr($_REQUEST[sitemap_url], 
		strpos($_REQUEST[sitemap_url], $SETTINGS[website])+
		strlen($SETTINGS[website]));
	//TODO why not use given $_REQUEST[sitemap_url]
	$SETTINGS[sitemap_file] = substr($SETTINGS[page_root],0,strlen($SETTINGS[page_root])) . $sitemap;
	$SETTINGS[sitemap_url] = $SETTINGS[website] . $sitemap;
	
	$changefreq = $_REQUEST[changefreq];
	if ($changefreq == "changefreq_dynamic") {
		$SETTINGS[changefreq_fixed] = false;
		$SETTINGS[changefreq_dynamic] = true;
	} elseif ($changefreq == "changefreq_fixed") {
		if ($_REQUEST[changefreq_fixed] != "") {
			$SETTINGS[changefreq_fixed] = $_REQUEST[changefreq_fixed]; 
			$SETTINGS[changefreq_dynamic] = false;
		} else {
			$LAYOUT->addWarning('Will use calculated change frequency.','No fixed change frequency set!');
			$SETTINGS[changefreq_fixed] = false;
			$SETTINGS[changefreq_dynamic] = true;			
		}
	}	
	
	if ($_REQUEST[priority] != "") {
		$SETTINGS[priority] = $_REQUEST[priority];
	} else {
		$SETTINGS[priority] = 0.5;
	}

	if ($_REQUEST[disallow_dir] != "") $SETTINGS[disallow_dir] = toArray($_REQUEST[disallow_dir]);
	if ($_REQUEST[disallow_file] != "") $SETTINGS[disallow_file] = toArray($_REQUEST[disallow_file]);

	// maybe try to open sitemap file
	
	// store settings in session
	$_SESSION[settings] = $SETTINGS;
	
	// write settings to file	
	// TODO implement this
	$res = storeSettings($SETTINGS, $SETTINGS[settings_file], "SETTINGS");
	if (!is_null($res)) {
		$LAYOUT->addWarning($res, 'Settings could not be written to file ' . $SETTINGS[settings_file] . '!');
	} else {
		$LAYOUT->addSuccess('', 'Settings written to file ' . $SETTINGS[settings_file] . '!');
	}
	return true;
}

?>